#!/bin/bash
# Author: Steven Shiau <steven _at_ nchc org tw>
# License: GPL

. /opt/drbl/sbin/drbl-conf-functions

# functions
USAGE() {
    echo "To start, restart or stop DRBL service in DRBL Live environment."
    echo "Usage:"
    echo "$0 {start|stop|restart}"
    echo "start   Start DRBL services now"
    echo "stop    Stop DRBL services now"
    echo "restart Restart DRBL services now"
    echo "Example: To start all DRBL services in this DRBL Live server"
    echo "$0 start"
}

# set locale for clients
set_locale_for_clients() {
  # Some of the code is referred to pakcage casper /usr/share/initramfs-tools/scripts/casper-bottom/14locales
  # We use the locale from server as the default one in client.
  if [ -e /etc/default/locale ]; then
      grep_file=/etc/default/locale
  elif [ -e /etc/environment ]; then # Old locales policy
      grep_file=/etc/environment
  fi
  if [ -n "${grep_file}" ]; then
      # use server's configured locale
      locale=$(grep -s 'LANG=' ${grep_file} | sed s/'LANG='// | tr -d '"' )
  fi
  if [ -z "$locale" ]; then
      # Set a default one
      locale=en_US.UTF-8
  fi
  LANG="${locale}"

  # write the setting in clients.
  for ih in $drblroot/*; do
    echo "LANG=${LANG}" > $ih/${grep_file}
  done
  echo "Since some config files are modified in template client, recreating template tarball for DRBL SSI..."
  $DRBL_SCRIPT_PATH/sbin/gen_ssi_files
} # end of set_locale_for_clients

check_if_root

#
if [ $# -ne 1 ]; then
  USAGE
  exit 1
fi

switch=$1

case "$switch" in
   "start"|"restart")
     # turn on tcpwrapper
     echo "Setting /etc/hosts.allow..."
     mv -f /etc/hosts.allow /etc/hosts.allow.orig
     client_IP_list="$($DRBL_SCRIPT_PATH/bin/get-client-ip-list)"
     client_IP_list="$(echo $client_IP_list)"   # convert to 1 line
     echo "ALL: $client_IP_list" > /etc/hosts.allow

     # add and start all services
     echo "Adding DRBL-related services..."
     $DRBL_SCRIPT_PATH/sbin/drbl-all-service add
     echo "Starting DRBL service.."
     $DRBL_SCRIPT_PATH/sbin/drbl-all-service restart

     # set locales for clients.
     set_locale_for_clients
     ;;
   "stop")
     # block all the clients.
     mv -f /etc/hosts.allow /etc/hosts.allow.orig2
     echo "Disabled by drbl-live.sh" > /etc/hosts.allow

     # stop and remove all services
     echo "Stop DRBL service.."
     $DRBL_SCRIPT_PATH/sbin/drbl-all-service stop
     echo "Removing DRBL-related services..."
     $DRBL_SCRIPT_PATH/sbin/drbl-all-service del
     ;;
esac
